#' Generate temporal pattern template at three levels
#'
#' @param time_series vector of dates
#' @param weekly_amplitude size of oscillations at weekly scale (default = 0)
#' @param seasonal_amplitude size of oscillations at seasonal scale (default = 0)
#' @param annual_amplitude size of oscillations at annual scale (default = 0)
#' @param weekly_noise variability at weekly scale (default = 0)
#' @param seasonal_noise variability at seasonal scale (default = 0)
#' @param annual_noise variability at annual scale (default = 0)
#'
#' @return data frame containing day of time series and corresponding pattern
#' @export
#'
#' @examples
#'
#' time_series <- set_timeline(4, "years")
#' pattern <- define_pattern_t(lubridate::ymd(time_series))
#'
#' ggplot2::ggplot(pattern, ggplot2::aes(x = date, y = pattern)) +
#' ggplot2::geom_line()
#'
define_pattern_t <- function(time_series,
                                     weekly_amplitude = 0,
                                     seasonal_amplitude = 0,
                                     annual_amplitude = 0,
                                     weekly_noise = 0,
                                     seasonal_noise = 0,
                                     annual_noise = 0) {
  time_series <- lubridate::ymd(time_series)
  series_length <- length(time_series)
  interval_length <- lubridate::interval(lubridate::ymd(min(time_series)),
                                       lubridate::ymd(max(time_series)))

  length_weeks <- interval_length |> lubridate::time_length("week")
  length_quarters <- length(unique(lubridate::quarter(time_series, with_year = TRUE)))
  length_years <- interval_length |> lubridate::time_length("year")

  weekly_function <- function(){
    weekly_pattern <- stats::rnorm(series_length,
                                   sin(2 * pi * lubridate::wday(time_series) / 7)  * weekly_amplitude,
                                   weekly_noise) #* weekly_amplitude
    weekly_pattern
  }

  seasonal_function <- function() {
    seasonal_pattern <- stats::rnorm(
      series_length,
      sin(2 * pi * lubridate::qday(time_series)/92) * seasonal_amplitude,
      seasonal_noise
    ) #* seasonal_amplitude
    seasonal_pattern
  }

  annual_function <- function(){
    annual_pattern <- stats::rnorm(series_length,
                                   sin(2 * pi * lubridate::yday(time_series)/365)  * annual_amplitude,
                                   annual_noise) #* annual_amplitude
    annual_pattern
  }

  if (length_weeks <= 1) {
    cli::cli_abort("Time series is too short to generate meaningful patterns")
  }
  if (length_weeks > 1 && length_quarters <=1) {
    cli::cli_warn(
      "One quarter or less of data, ignoring arguments for seasonal and annual patterns"
    )

    df <- data.frame(
      date = time_series,
      weekly_pattern = weekly_function()
    )
  }

  if (length_weeks > 1 && length_quarters > 1 && length_years  <= 1) {
    cli::cli_warn("Only one year or less of data, ignoring arguments for annual patterns")
    df <- data.frame(
      date = time_series,
      weekly_pattern = weekly_function(),
      seasonal_pattern = seasonal_function()
    )
  }
  if (length_years  > 1) {
    df <- data.frame(
      date = time_series,
      weekly_pattern = weekly_function(),
      seasonal_pattern = seasonal_function(),
      annual_pattern = annual_function()
    )
   }
  df <- df |>
    dplyr::mutate(pattern = rowSums(dplyr::across(tidyr::ends_with("pattern")))) |>
    dplyr::select(-tidyr::ends_with("_pattern"))
  df
}
