#' Sample spatial (and temporal) field
#'
#' @param field  dataframe with coordinates (`x`, `y`), an optional `date` column and a response `value` or `true_value` column
#' @param point_based if TRUE, points in field are sampled
#' @param areal_based default=FALSE if TRUE, whole field is sampled areally
#' @param n_samples Number of point samples (ignored if point_based is false)
#' @param x_range number of points from centroid on the x axis (for areal sampling)
#' @param y_range number of points from centroid on the y axis (for areal sampling)
#' @param sampling_error_points Sampling error for point based sampling (ignored if point_based is false)
#' @param sampling_error_areal default=NULL Sampling error for areal based sampling (ignored if areal_based is false)
#' @param regular If true, point samples are distributed evenly (ignored if point_based is false)
#' @param balanced default=FALSE. Should samples be spatially balanced (if TRUE), only applies when regular=FALSE
#'
#' @return dataframe containing sampled values
#' @export
#'
#' @examples
#'
#' time_df <- set_timeline(1, "month")
#' time_series <- define_pattern_t(time_df)
#' spatial_field <- define_pattern_s(20,20, autocorr = FALSE)
#' space_time <- simulate_spacetime(temporal_pattern = time_series,
#' field = spatial_field,
#' pattern_adherence = 0.1)
#'
#' field_samples <- sample_area(field = space_time, TRUE, TRUE, 20, 1, 1)
#'
sample_area <- function(field, point_based, areal_based = FALSE, n_samples,
                        x_range,
                        y_range,
                        sampling_error_points = 0, sampling_error_areal = 0,
                        regular = FALSE, balanced = FALSE){
  if(point_based == TRUE && areal_based == FALSE){
   sample_df <- sample_area_pointily(field,
                                     n_samples,
                                     sampling_error = sampling_error_points,
                                     balanced = balanced)
 }
  if(point_based == FALSE && areal_based == TRUE){
    sample_df <- sample_area_areally(field, n_samples,
                                     x_range,
                                     y_range,
                                     sampling_error = sampling_error_areal)
  } else{
    point_sample <- sample_area_pointily(field,
                                         n_samples,
                                         sampling_error = sampling_error_points,
                                         regular = regular,
                                         balanced = balanced)
    areal_sample <- sample_area_areally(field, n_samples,  x_range,
                                        y_range,
                                        sampling_error = sampling_error_areal)
    if (!"date" %in% colnames(field)) {
     sample_df <- cbind(point_sample, areal_sample)
    }else{
      sample_df <- dplyr::inner_join(point_sample, areal_sample)
    }
  }
  sample_df
}


