#' Take area samples in field
#'
#' @param field dataframe with coordinates (`x`, `y`), an optional `date` column and a response `value` column
#' @param n_samples number of samples to be taken - note this is approximate depending on dimension divisibility
#' @param x_range number of points from centroid on the x axis
#' @param y_range number of points from centroid on the y axis
#' @param sampling_error noise around each sample
#' @param regular default=TRUE. Should samples be taken on regular grid (if TRUE), or randomly (if FALSE)
#' @param balanced default=FALSE. Should samples be spatially balanced (if TRUE), only applies when regular=FALSE
#'
#' @return data frame with sampled values
#' @export
#'
#' @examples
#'
#' field <- define_pattern_s(20, 20, TRUE, 4, 1)
#' field_sample <- sample_area_areally(field, 10,2,2, 0.1, regular = FALSE)
#'
sample_area_areally <-
  function(field,
           n_samples,
           x_range,
           y_range,
           sampling_error,
           regular = TRUE,
           balanced = FALSE) {
    dim_x <- max(field$x) - min(field$x) + 1
    dim_y <- max(field$y) - min(field$y) + 1
    if (regular == TRUE && balanced == TRUE) {
      cli::cli_warn("Regular and balance both set to TRUE. Balance argument ignored.")
    }

    if (n_samples >= dim_x * dim_y) {
      cli::cli_warn(
        "Number of samples is equal to or greater than the number of grid cells. Instead returning full area with sampling error."
      )
      if (!"date" %in% colnames(field)) {
        areal_sample_df <- field |>
          dplyr::mutate(point_sample = .data$value + stats::rnorm(nrow(field), 0, sampling_error))
      } else {
        areal_sample_df <- field |>
          dplyr::group_by(date) |>
          dplyr::mutate(point_sample = .data$value + stats::rnorm(nrow(field), 0, sampling_error))
      }
    }

    if (regular == TRUE) {
      nx <- round(sqrt(n_samples * (dim_x / dim_y)))
      ny <- round(sqrt(n_samples * (dim_y / dim_x)))
      dist_between_points_x <- floor((dim_x - 1) / (nx - 1))
      dist_between_points_y <- floor((dim_y - 1) / (ny - 1))

      grid <-
        expand.grid(x = round(seq(1, dim_x, dist_between_points_x)),
                    y = round(seq(1, dim_y, dist_between_points_y)))

      if (!"date" %in% colnames(field)) {
        areal_sample_df <- grid
        areal_sample_df$area_value <- NA

        for (i in 1:nrow(grid)) {
          temp_df <-
            expand.grid(
              x = seq(
                areal_sample_df$x[i] - x_range,
                areal_sample_df$x[i] + x_range,
                1
              ),
              y = seq(
                areal_sample_df$y[i] - y_range,
                areal_sample_df$y[i] + y_range,
                1
              )
            )

          temp_df_value <- temp_df |>
            dplyr::left_join(field, by = dplyr::join_by("x", "y"))

          areal_sample_df$area_value[i] <-
            mean(temp_df_value$value, na.rm = T) + stats::rnorm(1, 0, sampling_error)
        }

      } else {
        temp_df <- expand.grid(
          x = seq(grid$x[1] - x_range, grid$x[1] + x_range, 1),
          y = seq(grid$y[1] - y_range, grid$y[1] + y_range, 1)
        )

        temp_df_value <- temp_df |>
          dplyr::left_join(field, by = dplyr::join_by("x", "y"))

        areal_sample_df <-
          stats::aggregate(temp_df_value, value ~ date, mean, na.rm = T)
        areal_sample_df$area_value <-
          areal_sample_df$value + stats::rnorm(1, 0, sampling_error)
        areal_sample_df$x <- grid$x[1]
        areal_sample_df$y <- grid$y[1]


        for (i in 2:nrow(grid)) {
          temp_df <- expand.grid(
            x = seq(grid$x[i] - x_range, grid$x[i] + x_range, 1),
            y = seq(grid$y[i] - y_range, grid$y[i] +
                      y_range, 1)
          )

          temp_df_value <- temp_df |>
            dplyr::left_join(field, by = dplyr::join_by("x", "y"))

          temp_df_mean <-
            stats::aggregate(temp_df_value, value ~ date, mean, na.rm = T)
          temp_df_mean$area_value <-
            temp_df_mean$value + stats::rnorm(1, 0, sampling_error)
          temp_df_mean$x <- grid$x[i]
          temp_df_mean$y <- grid$y[i]

          areal_sample_df <- rbind(areal_sample_df, temp_df_mean)
        }
      }
    }# end regular=TRUE

    if (regular == FALSE && balanced == TRUE) {
      if (!"date" %in% colnames(field)) {
        field_m <- as.matrix(dplyr::select(field, -.data$value))
        studyarea <-
          matrix(c(0, 0, 0, dim_x, dim_y, dim_x, dim_y, 0),
                 ncol = 2,
                 byrow = TRUE)

        areal_sample_df <- field[MBHdesign::quasiSamp(n = n_samples,
                                                      study.area = studyarea,
                                                      potential.sites = field_m)$ID,]

        #transform into sf polygon and check for overlaps - resample if it overlaps
        sfg.list <- apply(areal_sample_df, 1, function(i, j) {
          sf::st_polygon(list(matrix(
            c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
              i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
            ncol = 2
          )))
        }) |>
          sf::st_as_sfc() |>
          sf::st_as_sf()

        temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))

        while(length(temp)>0){
          areal_sample_df <- field[MBHdesign::quasiSamp(n = n_samples,
                                                        study.area = studyarea,
                                                        potential.sites = field_m)$ID,]

          sfg.list <- apply(areal_sample_df, 1, function(i, j) {
            sf::st_polygon(list(matrix(
              c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
                i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
              ncol = 2
            )))
          }) |>
            sf::st_as_sfc() |>
            sf::st_as_sf()
          temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))
        }

        areal_sample_df$area_value <- NA

        for (i in 1:nrow(areal_sample_df)) {
          temp_df <-
            expand.grid(
              x = seq(
                areal_sample_df$x[i] - x_range,
                areal_sample_df$x[i] + x_range,
                1
              ),
              y = seq(
                areal_sample_df$y[i] - y_range,
                areal_sample_df$y[i] + y_range,
                1
              )
            )

          temp_df_value <- temp_df |>
            dplyr::left_join(field, by = dplyr::join_by("x", "y"))

          areal_sample_df$area_value[i] <-
            mean(areal_sample_df$value, na.rm = T) + stats::rnorm(1, 0, sampling_error)
        }

      } else {
        field_with_row <- field |>
          dplyr::group_by(date) |>
          dplyr::mutate(row = dplyr::row_number())


        splitfield <- split(field, field$date)

        splitfield_m <-
          as.matrix(dplyr::select(splitfield[[1]], -c(date, .data$value)))

        studyarea <-
          matrix(c(0, 0, 0, dim_x, dim_y, dim_x, dim_y, 0),
                 ncol = 2,
                 byrow = TRUE)

        sampled_points <- MBHdesign::quasiSamp(n = n_samples,
                                               study.area = studyarea,
                                               potential.sites = splitfield_m)$ID

        grid <- as.data.frame(splitfield_m[sampled_points, ])

        #transform into sf polygon and check for overlaps - resample if it overlaps
        sfg.list <- apply(grid, 1, function(i, j) {
          sf::st_polygon(list(matrix(
            c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
              i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
            ncol = 2
          )))
        }) |>
          sf::st_as_sfc() |>
          sf::st_as_sf()

        temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))

        while(length(temp)>0){
          sampled_points <- MBHdesign::quasiSamp(n = n_samples,
                                                 study.area = studyarea,
                                                 potential.sites = splitfield_m)$ID

          grid <- as.data.frame(splitfield_m[sampled_points, ])


          sfg.list <- apply(grid, 1, function(i, j) {
            sf::st_polygon(list(matrix(
              c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
                i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
              ncol = 2
            )))
          }) |>
            sf::st_as_sfc() |>
            sf::st_as_sf()
          temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))
        }


        temp_df <-
          expand.grid(
            x = seq(grid$x[1] - x_range, grid$x[1] + x_range, 1),
            y = seq(grid$y[1] - y_range, grid$y[1] + y_range, 1)
          )

        temp_df_value <- temp_df |>
          dplyr::left_join(field, by = dplyr::join_by("x", "y"))

        areal_sample_df <-
          stats::aggregate(temp_df_value, value ~ date, mean, na.rm = T)
        areal_sample_df$area_value <-
          areal_sample_df$value + stats::rnorm(1, 0, sampling_error)
        areal_sample_df$x <- grid$x[1]
        areal_sample_df$y <- grid$y[1]


        for (i in 2:nrow(grid)) {
          temp_df <- expand.grid(
            x = seq(grid$x[i] - x_range, grid$x[i] + x_range, 1),
            y = seq(grid$y[i] - y_range, grid$y[i] +
                      y_range, 1)
          )

          temp_df_value <- temp_df |>
            dplyr::left_join(field, by = dplyr::join_by("x", "y"))

          temp_df_mean <-
            stats::aggregate(temp_df_value, value ~ date, mean, na.rm = T)
          temp_df_mean$area_value <-
            temp_df_mean$value + stats::rnorm(1, 0, sampling_error)
          temp_df_mean$x <- grid$x[i]
          temp_df_mean$y <- grid$y[i]

          areal_sample_df <- rbind(areal_sample_df, temp_df_mean)
        }
      }
    } #end regular == FALSE && balanced == TRUE

    if (regular == FALSE && balanced == FALSE) {
      if (!"date" %in% colnames(field)) {
        areal_sample_df <- field |>
          dplyr::sample_n(n_samples)

        #transform into sf polygon and check for overlaps - resample if it overlaps
        sfg.list <- apply(areal_sample_df, 1, function(i, j) {
          sf::st_polygon(list(matrix(
            c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
              i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
            ncol = 2
          )))
        }) |>
          sf::st_as_sfc() |>
          sf::st_as_sf()

        temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))

        while(length(temp)>0){
          areal_sample_df <- field |>
            dplyr::sample_n(n_samples)

          sfg.list <- apply(areal_sample_df, 1, function(i, j) {
            sf::st_polygon(list(matrix(
              c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
                i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
              ncol = 2
            )))
          }) |>
            sf::st_as_sfc() |>
            sf::st_as_sf()
          temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))
        }

        areal_sample_df$area_value <- NA


        for (i in 1:nrow(areal_sample_df)) {
          temp_df <-
            expand.grid(
              x = seq(
                areal_sample_df$x[i] - x_range,
                areal_sample_df$x[i] + x_range,
                1
              ),
              y = seq(
                areal_sample_df$y[i] - y_range,
                areal_sample_df$y[i] + y_range,
                1
              )
            )

          temp_df_value <- temp_df |>
            dplyr::left_join(field, by = dplyr::join_by("x", "y"))
          areal_sample_df$area_value[i] <-
            mean(areal_sample_df$value, na.rm = T) + stats::rnorm(1, 0, sampling_error)
        }

      } else {

        grid <- data.frame(unique(field[,1:2]) |>
                             dplyr::sample_n(n_samples))

        #transform into sf polygon and check for overlaps - resample if it overlaps
        sfg.list <- apply(grid, 1, function(i, j) {
          sf::st_polygon(list(matrix(
            c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
              i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
            ncol = 2
          )))
        }) |>
          sf::st_as_sfc() |>
          sf::st_as_sf()

        temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))

        while(length(temp)>0){
          grid <- data.frame(unique(field[,1:2]) |>
                               dplyr::sample_n(n_samples))

          sfg.list <- apply(grid, 1, function(i, j) {
            sf::st_polygon(list(matrix(
              c(i[1] + x_range, i[1] - x_range, i[1] - x_range, i[1] + x_range, i[1] + x_range,
                i[2] + y_range, i[2] + y_range, i[2] - y_range, i[2] - y_range, i[2] + y_range),
              ncol = 2
            )))
          }) |>
            sf::st_as_sfc() |>
            sf::st_as_sf()
          temp<-unlist(sf::st_overlaps(sfg.list,sparse=TRUE))
        }

temp_df <-
          expand.grid(
            x = seq(grid$x[1] - x_range, grid$x[1] + x_range, 1),
            y = seq(grid$y[1] - y_range, grid$y[1] + y_range, 1)
          )

        temp_df_value <- temp_df |>
          dplyr::left_join(field, by = dplyr::join_by("x", "y"))

        areal_sample_df <-
          stats::aggregate(temp_df_value, value ~ date, mean, na.rm = T)
        areal_sample_df$area_value <-
          areal_sample_df$value + stats::rnorm(1, 0, sampling_error)
        areal_sample_df$x <- grid$x[1]
        areal_sample_df$y <- grid$y[1]


        for (i in 2:nrow(grid)) {
          temp_df <- expand.grid(
            x = seq(grid$x[i] - x_range, grid$x[i] + x_range, 1),
            y = seq(grid$y[i] - y_range, grid$y[i] +
                      y_range, 1)
          )

          temp_df_value <- temp_df |>
            dplyr::left_join(field, by = dplyr::join_by("x", "y"))

          temp_df_mean <-
            stats::aggregate(temp_df_value, value ~ date, mean, na.rm = TRUE)
          temp_df_mean$area_value <-
            temp_df_mean$value + stats::rnorm(1, 0, sampling_error)
          temp_df_mean$x <- grid$x[i]
          temp_df_mean$y <- grid$y[i]

          areal_sample_df <- rbind(areal_sample_df, temp_df_mean)
        }

      }
    } #end regular == FALSE && balanced == FALSE

    areal_sample_df
  }
