#' Take point samples in field
#'
#' @param field dataframe with coordinates (`x`, `y`), an optional `date` column and a response `value` column
#' @param n_samples number of samples to be taken - note this is approximate depending on dimension divisibility
#' @param sampling_error noise around each sample
#' @param regular default=TRUE. Should samples be taken on regular grid (if TRUE), or randomly (if FALSE)
#' @param balanced default=FALSE. Should samples be spatially balanced (if TRUE), only applies when regular=FALSE
#'
#' @return data frame with sampled values
#' @export
#'
#' @examples
#'
#' field <- define_pattern_s(20, 20, TRUE, 4, 1)
#' field_sample <- sample_area_pointily(field, 10, 0.1, regular = FALSE)
#'
sample_area_pointily <-
  function(field,
           n_samples,
           sampling_error,
           regular = TRUE,
           balanced = FALSE) {
    dim_x <- max(field$x) - min(field$x) + 1
    dim_y <- max(field$y) - min(field$y) + 1
    if (regular == TRUE && balanced == TRUE) {
      cli::cli_warn(
        "Regular and balance both set to TRUE. Balance argument ignored."
      )
      }
      if (n_samples >= dim_x * dim_y) {
      cli::cli_warn(
        "Number of samples is equal to or greater than the number of grid cells. Instead returning full area with sampling error."
      )
      if (!"date" %in% colnames(field)) {
        point_sample_df <- field |>
          dplyr::mutate(point_sample =.data$value + stats::rnorm(nrow(field), 0, sampling_error))
      } else {
        point_sample_df <- field |>
          dplyr::group_by(date) |>
          dplyr::mutate(point_sample =.data$value + stats::rnorm(nrow(field), 0, sampling_error))
      }
    }

    if (regular == TRUE) {
      nx <- round(sqrt(n_samples*(dim_x / dim_y)))
      ny <- round(sqrt(n_samples*(dim_y / dim_x)))
      dist_between_points_x <- floor((dim_x - 1) / (nx-1))
      dist_between_points_y <- floor((dim_y - 1) / (ny-1))
      grid <-
        expand.grid(x = round(seq(1, dim_x, dist_between_points_x)),
                    y = round(seq(1, dim_y, dist_between_points_y)))

       if (!"date" %in% colnames(field)) {
        point_sample_df <- grid |>
          dplyr::left_join(field) |>
          dplyr::mutate(point_sample = .data$value + stats::rnorm(nrow(grid), 0, sampling_error))
      } else {
        point_sample_df <- grid |>
          dplyr::left_join(field) |>
          dplyr::group_by(date) |>
          dplyr::mutate(point_sample = .data$value + stats::rnorm(nrow(grid), 0, sampling_error))
      }
    } #end regular

      if (regular == FALSE && balanced == TRUE) {
        if (!"date" %in% colnames(field)) {
          field_m <- as.matrix(dplyr::select(field,-.data$value))
          studyarea <-
            matrix(c(0, 0, 0, dim_x, dim_y, dim_x, dim_y, 0),
                   ncol = 2,
                   byrow = TRUE)
          point_sample_df <-
            field[MBHdesign::quasiSamp(
              n = n_samples,
              study.area = studyarea,
              potential.sites = field_m
            )$ID, ] |>
            dplyr::mutate(point_sample = .data$value + stats::rnorm(n_samples, 0, sampling_error))
        } else {
          field_with_row <- field |>
            dplyr::group_by(date) |>
            dplyr::mutate(row = dplyr::row_number())
          splitfield <- split(field, field$date)
          splitfield_m <-
            as.matrix(dplyr::select(splitfield[[1]],-c(date, .data$value)))
          studyarea <-
            matrix(c(0, 0, 0, dim_x, dim_y, dim_x, dim_y, 0),
                   ncol = 2,
                   byrow = TRUE)
          sampled_points <-
            MBHdesign::quasiSamp(
              n = n_samples,
              study.area = studyarea,
              potential.sites = splitfield_m
            )$ID
          point_sample_df <-
            field[field_with_row$row %in% sampled_points, ] |>
            dplyr::mutate(point_sample = .data$value + stats::rnorm(n_samples, 0, sampling_error))
        }
      } # end random balance

    if (regular == FALSE && balanced == FALSE) {
        if (!"date" %in% colnames(field)) {
          grid <- data.frame(field |>
                               dplyr::sample_n(n_samples))[,1:2]

          point_sample_df <- grid |>
            dplyr::sample_n(n_samples) |>
            dplyr::mutate(point_sample = .data$value + stats::rnorm(n_samples, 0, sampling_error))
        } else {

          grid <- data.frame(unique(field[,1:2]) |>
                               dplyr::sample_n(n_samples))

          point_sample_df <- grid |>
            dplyr::left_join(field) |>
            dplyr::group_by(date) |>
            dplyr::mutate(point_sample = .data$value + stats::rnorm(nrow(grid), 0, sampling_error))
        }
      }
    point_sample_df
  }
