#' Apply temporal pattern in space
#'
#' @param temporal_pattern pattern generated by `define_pattern_t`
#' @param field spatial field generated by
#' @param pattern_adherence how closely do cells adhere to the defined pattern (large values indicate more noise)
#'
#' @return space-time dataframe
#' @importFrom rlang .data
#' @export
#'
#' @examples
#'
#' time_df <- set_timeline(1, "month")
#' time_series <- define_pattern_t(time_df)
#' spatial_field <- define_pattern_s(20,20, autocorr = FALSE)
#' space_time <- simulate_spacetime(temporal_pattern = time_series,
#' field = spatial_field,
#' pattern_adherence = 0.1)
#'
#' ggplot2::ggplot(space_time, ggplot2::aes(x = x, y = y, fill = value)) +
#' ggplot2::geom_raster() +
#' ggplot2::facet_wrap(~date)
#'
#' space_time$loc_id <- paste(space_time$x, space_time$y, sep = "_")
#'
#' space_time_red <- space_time |> dplyr::filter(loc_id %in% sample(unique(space_time$loc_id), 5))
#'
#' ggplot2::ggplot(space_time_red, ggplot2::aes(x = date, y = value, colour = interaction(x,y))) +
#'   ggplot2::geom_line() +
#'   ggplot2::theme(legend.position = "none")
#'
simulate_spacetime <- function(temporal_pattern,
                          field,
                          pattern_adherence = 0) {
  combi_df <- tidyr::expand_grid(field, temporal_pattern)
  combi_df <- combi_df |>
    dplyr::rename(starting_value = .data$value) # new name = old name
  combi_df$value <- stats::rnorm(nrow(combi_df),
                               (combi_df$starting_value + combi_df$pattern),
                                                pattern_adherence)
  combi_df <- combi_df |>
    dplyr::select(-c(.data$starting_value, .data$pattern))
  combi_df
}
