packages<-c("ggplot2", "shiny","shinydashboard")
install.packages(setdiff(packages, rownames(installed.packages())))

if(!("ascot" %in% rownames(installed.packages())))
{devtools::install_git("https://gitlab.bioss.ac.uk/ukceh-bioss-framework/dev/ascot.git")}

library(ascot)
library(shiny)
library(shinydashboard)
library(ggplot2)

###User interface
ui <- dashboardPage(
  dashboardHeader(title = "Simulating spatio-temporal data and sampling from the data using the ASCOT package", titleWidth = 1000),
  dashboardSidebar(
    sidebarMenu(
      menuItem("Simulate spatio-temporal data", tabName = "sims", icon = icon("dashboard")),
      menuItem("Sampling designs", tabName = "sampling", icon = icon("th"))
    )#close side bar menu
  ),#close sidebar
  dashboardBody(
    tabItems(
      tabItem(tabName = "sims",
              fluidPage(
                sidebarLayout(
                  sidebarPanel(
                    #timeseries
                    h2("Create timeseries"),
                    helpText("Please select the duration of your study period in weeks, months or years. Note that regardless of this choice, the resulting timeseries will always be daily observations."),
                    selectInput("time", label = "Select time series unit", choices = c("week", "month", "year"), selected = "month"),
                    sliderInput("dur",label = "Select time series duration",min = 1, max = 12, value = 2, step = 1, round=TRUE, ticks = FALSE),
                    #temporal data
                    h2("Temporal simulation parameters"),
                    helpText("Creates a temporal dataset for the study period. Please select if you want a weekly, seasonal and/or annual patterns in the data (note that seasonal variation requires a minimum of four months, and annual variation requires a minimum of two years). There are two components to generate the data: amplitude and noise. The amplitude corresponds to the distance from 0, while noise is how much variation there is away from the given temporal trend."),
                    fluidRow(
                      column(6, sliderInput("week_amp",label = "Week amplitude", min = 0, max = 100, value = 1)),
                      column(6,sliderInput("week_noise", label = "Week noise", min = 0, max = 100, value = 0)),
                      column(6,sliderInput("season_amp",label = "Season amplitude", min = 0, max = 100, value = 0)),
                      column(6,sliderInput("season_noise", label = "Seasonal noise", min = 0, max = 100, value = 0)),
                      column(6,sliderInput("annual_amp", label = "Annual amplitude",  min = 0, max = 100, value = 0)),
                      column(6,sliderInput("annual_noise", label = "Annual noise", min = 0, max = 100, value = 0))),

                    #spatial data
                    h2("Spatial simulation parameters"),
                    helpText("Creates a spatial dataset. Please select the the size of the spatial area by increasing or decreasing x and y, and check the box if you would like to include spatial autocorrelation. When spatial autocorrelation is included, the spatial extent of autocorrelation (range) and variance (amplitude) can be selected."),
                    fluidRow(
                      column(6, sliderInput("x", label = "x", min = 10, max = 50, value = 20)),
                      column(6, sliderInput("y",label = "y", min = 10, max = 50, value = 20)),
                      checkboxInput("spatial_autocorr", label = "Spatial autocorrelation", value = FALSE),
                      # Conditional panel to show slider based on condition
                      conditionalPanel(
                        condition = "input.spatial_autocorr",
                        column(6,sliderInput("range", "Range", min = 0, max = 20, value = 1)),
                        column(6,sliderInput("amplitude", "Amplitude", min = 1, max = 20, value = 1))
                      )#close conditional Panel
                    ), #close fluidrow

                    #spatial-temporal data
                    h2("Spatial-temporal simulation parameters"),
                    helpText("Creates a spatio-temporal dataset by combining the spatial and temporal datasets. You are able to increase or decrease how consistent the pattern is adhered to by altering the pattern adherence value. The higher the value, the higher the variation in the data."),
                    sliderInput("adh",label = "Pattern adherence", min = 0, max = 20, value = 1)
                  ), # close sidebarPanel

                  mainPanel(
                    tabsetPanel(type = "tabs",
                                tabPanel("Plots",h2("Spatial visualisation"),
                                         helpText("Daily spatial plots. Please select how many days to display (currently limited to 12 days). To see the temporal variation of a single point please click on that point."),
                                         sliderInput("nplots",label = "Number of plots to show", min = 1, max = 12, value = 1),
                                         plotOutput('spatial', click = "plot_click"),
                                         h2("Temporal visualisation"),
                                         helpText("Timeseries plot of the selected point."),
                                         plotOutput('temp')),
                                tabPanel("Dataset",  dataTableOutput("true_dataset"))
                    )#close tabs
                  )#close main panel
                )#close sidebar layout
              )# close fluidpage
      ),#close tab item (sims)
      tabItem(tabName = "sampling",
              fluidPage(
                sidebarLayout(
                  sidebarPanel(
                    #timeseries
                    h2("Sampling design parameters"),
                    helpText("Please select if you want to simulate point sampling or area sampling and the type of sampling design (random, balanced or regular). For random sampling, pixels are randomly selected; balanced sampling creates a 'spatial balanced' design to avoid clustering of samples; and for regular sampling, the selected pixels are evenly distributed."),
                    selectInput("scale", "Select sampling scale",choices= c("Point", "Area"), selected = "Point", multiple = FALSE),
                    selectInput("design", "Select type of sampling", choices= c("Random", "Balanced", "Regular"), selected = "Random", multiple = FALSE),
                    fluidRow(
                      column(6,sliderInput("nsample", "Number of samples", min = 2, max = 10, value = 3)),
                      column(6,sliderInput("error", "Sampling error", min = 0, max = 5, value = 0.5,step = 0.1, round = FALSE)),
                      conditionalPanel(
                        condition = "input.scale=='Area'",
                        column(6,sliderInput("xrange", "Distance from centroid to perimeter on x axis", min = 1, max = 10, value = 1)),
                        column(6,sliderInput("yrange", "Distance from centroid to perimeter on y axis", min = 1, max = 10, value = 1))
                      )#close conditional Panel
                    ), #close fluid row
                    downloadButton("sampledata", "Download dataset")
                  ),#close sidebarpanel
                  mainPanel(
                    tabsetPanel(type = "tabs",
                                tabPanel("Plots", h2("Spatial visualisation"),
                                         helpText("Daily spatial plots with sampled areas or points. Please select how many days to display (currently limited to 12 days)."),
                                         sliderInput("nplots_sample",label = "Number of plots to show", min = 1, max = 12, value = 1),
                                         plotOutput('spatial_sample'),
                                         h2("Temporal visualisation"),
                                         helpText("Timeseries of the sampled points or areas."),
                                         plotOutput('temp_sample')),
                                tabPanel("Dataset",  dataTableOutput("samples_dataset"))
                    )#close tabs
                  )#close main panel
                )#close sidebarlayout
              )#close fluid page
      )#close tab item
    )#close tab items
  )#close dashboard body
)#close dashboard page

server <- function(input, output, session) {

  observe({
    if(input$time=="week"){
    updateSliderInput(session, "dur", value = 2, min = 2, max = 52, step = 1)}

    if(input$time=="year"){
      updateSliderInput(session, "dur", value = 1, min = 1, max = 3)}

    if(input$time=="month"){
      updateSliderInput(session, "dur", value = 1, min = 1, max = 12)}
  })

  seed <- sample(1:2^15, 1)
  set.seed(seed)

  ### Code for simulation

  # Reactive value to store clicked point information
  clickedPoint <- reactiveVal(NULL)

  #create spatio-temporal data
  space_time<-reactive({

    #set period
    time_df<-set_timeline(input$dur,input$time)

    #create pattern
    temporal_pattern <- define_pattern_t( time_series = time_df,
                                          weekly_amplitude = input$week_amp,
                                          seasonal_amplitude = input$season_amp,
                                          annual_amplitude = input$annual_amp,
                                          weekly_noise = input$week_noise,
                                          seasonal_noise = input$season_noise,
                                          annual_noise = input$annual_noise)

    # Define the spatial field
    spatial_field <- define_pattern_s(input$x,
                                      input$y,
                                      autocorr = input$spatial_autocorr,
                                      range = input$range,
                                      amplitude = input$amplitude)

    # Simulate spatial-temporal data
    temp_space_time <- simulate_spacetime(temporal_pattern = temporal_pattern,
                                          field = spatial_field,
                                          pattern_adherence = input$adh)
    return(temp_space_time)
  })

  # Create a spatial plot per day
  output$spatial <- renderPlot({
    time_df <- set_timeline(input$dur, input$time)[1:input$nplots]
    temp_df <- space_time()
    temp_df <- temp_df[which(temp_df$date %in% time_df), ]

    # Plot the spatial data
    ggplot(temp_df, aes(x = x, y = y, fill = value)) +
      geom_raster(hjust = 0, vjust = 0) +
      labs(fill = "Value") +
      xlab("X coordinates") +
      ylab("Y coordinates") +
      coord_equal(expand = FALSE) +
      facet_wrap(~date) +
      scale_fill_viridis_c() +
      theme_bw()
  })

  # Handle click event and update reactive value
  observeEvent(input$plot_click, {
    click_data <- input$plot_click
    if (!is.null(click_data)) {
      clickedPoint(click_data)
    }
  })

  #get timeseries for each point
  point_data<-reactive({
    click_info <- clickedPoint()
    temp_stdata<- space_time()

    if (!is.null(click_info)) {
      clicked_x <- click_info$x
      clicked_y <- click_info$y

      # Calculate Euclidean distance
      distances <- sqrt((temp_stdata$x - clicked_x)^2 + (temp_stdata$y - clicked_y)^2)

      # Find the index of the nearest point
      loc_id<-paste(temp_stdata$x, temp_stdata$y, sep = "_")

      nearest_index <- loc_id[which.min(distances)]
      # Extract information about the nearest point
      nearest_data <- temp_stdata[loc_id==nearest_index, ]

      return(nearest_data)
    } else {
      return(NULL)
    }
  })

  # Plot time series for each point
  output$temp <- renderPlot({
    if(length(point_data())>0){
      ggplot(point_data(), aes(x=date, y=value))+
        geom_point()+
        geom_path()+
        theme_bw()}
  })

  output$true_dataset <- renderDataTable(space_time())

  ### Code for sampling

  #create spatio-temporal data
  sampling_data<-reactive({

    #point sampling random
    if(input$scale=="Point" && input$design=="Random"){
      temp_sample<-sample_area_pointily(field= space_time(), n_samples=input$nsample, sampling_error=input$error, regular = FALSE, balanced = FALSE)}

    #point sampling balanced
    if(input$scale=="Point" && input$design=="Balanced"){
      temp_sample<-sample_area_pointily(field= space_time(), n_samples=input$nsample, sampling_error=input$error, regular = FALSE, balanced = TRUE)}

    #point sampling regular
    if(input$scale=="Point" && input$design=="Regular"){
      temp_sample<-sample_area_pointily(field= space_time(), n_samples=input$nsample, sampling_error=input$error, regular = TRUE)}

    #area sampling random
    if(input$scale=="Area" && input$design=="Random"){
      temp_sample<-sample_area_areally(field= space_time(), n_samples=input$nsample, sampling_error=input$error, x_range = input$xrange,y_range = input$yrange, regular = FALSE, balanced = FALSE)}

    #area sampling balanced
    if(input$scale=="Area" && input$design=="Balanced"){
      temp_sample<-sample_area_areally(field= space_time(), n_samples=input$nsample, sampling_error=input$error, x_range = input$xrange,y_range = input$yrange, regular = FALSE, balanced = TRUE)}

    #area sampling regular
    if(input$scale=="Area" && input$design=="Regular"){
      temp_sample<-sample_area_areally(field= space_time(), n_samples=input$nsample, sampling_error=input$error, x_range = input$xrange,y_range = input$yrange, regular = TRUE)}

    return(temp_sample)
  })


  output$samples_dataset <- renderDataTable(sampling_data())

  # Create a spatial plot per day
  output$spatial_sample <- renderPlot({

    time_df <- set_timeline(input$dur, input$time)[1:input$nplots_sample]
    temp_df <- space_time()
    temp_df <- temp_df[which(temp_df$date %in% time_df), ]

    temp_sampling<-sampling_data()
    temp_sampling <- temp_sampling[which(temp_sampling$date %in% time_df), ]

    if(input$scale=="Point"){
      # Plot the spatial data
      temp_plot<- ggplot(temp_df, aes(x = x, y = y, fill = value)) +
        geom_raster() +
        labs(fill = "Value") +
        xlab("X coordinates") +
        ylab("Y coordinates") +
        geom_point(data=temp_sampling, aes(x=x,y=y), color="white", size=2) +
        coord_equal(expand = FALSE) +
        facet_wrap(~date) +
        scale_fill_viridis_c() +
        theme_bw()}

    if(input$scale=="Area"){
      # Plot the spatial data
      temp_plot<- ggplot(temp_df, aes(x = x, y = y, fill = value)) +
        geom_raster(alpha=0.8, hjust = 0, vjust = 0) +
        labs(fill = "Value") +
        xlab("X coordinates")+
        ylab("Y coordinates") +
        geom_rect(data=temp_sampling, mapping=aes(xmin=x-input$xrange, xmax=x+input$xrange, ymin=y-input$yrange, ymax=y+input$yrange, fill=area_value), color="black") +
        facet_wrap(~date)+
        coord_equal(expand=FALSE, ylim=c(0,input$y), xlim=c(0,input$x))+
        scale_fill_viridis_c()+
        theme_bw()}

    return(temp_plot)

  })


  # Plot time series for each point
  output$temp_sample <- renderPlot({
      ggplot(sampling_data(), aes(x=date, y=value, group=paste0(x,"-",y)))+
        geom_point()+
        geom_path()+
        theme_bw()
  })


output$sampledata <- downloadHandler(
  filename <- function(){
    paste("Simulated_dataset_",Sys.Date(),".RData")
  },
  content = function(file) {
    data<-list()
    data[['seed']]<-seed
    data[['true_data']]<-list(data = space_time(),
                    time_period = paste(input$dur, input$time),
                    weekly_amp = input$week_amp,
                    weekly_noise = input$week_noise,
                    seasonal_amp = input$season_amp,
                    seasonal_noise = input$season_noise,
                    annual_amp = input$annual_amp,
                    annual_noise = input$annual_noise,
                    x = input$x,
                    y= input$y,
                    pattern_adherence= input$adh,
                    autocorr = input$spatial_autocorr)

    if(input$spatial_autocorr=="TRUE"){
        data[['true_data']]<-append(data[['true_data']], list(range= input$range,
                                        amplitude = input$amplitude))
    }

    data[['sampled_data']]<- list(data = sampling_data(),
                                  sampling_scale=input$scale,
                                  sampling_design=input$design,
                                  nsamples = input$nsample,
                                  sampling_error=input$error)

    if(input$scale=="Area"){
      data[['sampled_data']]<-append(data[['sampled_data']], list(xrange= input$xrange,
                                                            yrange=input$yrange))
    }

    save(data, file = file)
  })

}
shinyApp(ui, server)





