## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>", fig.height = 6, fig.width = 8, cache = TRUE
)

## ----message=FALSE------------------------------------------------------------
library(ascot) # Our package for simulating space-time patterns
library(dplyr) # Handy package for general data manipulation
library(ggplot2) # Visualisation
library(lubridate) # Nice way to work with date data
library(mgcv) # The GAMs package we will use for modelling

## ----echo=TRUE----------------------------------------------------------------
# Determine duration of time series
time_df <- set_timeline(1, "month")
# Set time series pattern
time_series <- define_pattern_t(time_df,
                                weekly_amplitude = 1,
                                weekly_noise = 0)

# Visualise
ggplot(time_series, aes(x = date, y = pattern)) +
  geom_point() + 
  geom_line() +
  theme_bw()

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Generate Field
#  spatial_field <- define_pattern_s(20, 20, autocorr = TRUE, range = 5)
#  
#  # Visualise
#  ggplot(spatial_field, aes(x = x, y = y, fill = value)) +
#    geom_raster() +
#    theme_bw()

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Simulate from spatial and temporal patterns
#  truth <- simulate_spacetime(temporal_pattern = time_series,
#                                   field = spatial_field,
#                                   pattern_adherence = 0)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Generate vector of random dates
#  random_dates <- truth$date %>% unique() %>% sample(9)
#  # Sample those dates
#  truth_rand_times <- truth %>% filter(date %in% random_dates)
#  # Visualise
#  ggplot(truth_rand_times, aes(x = x, y = y, fill = value)) +
#    geom_raster() +
#    facet_wrap(vars(date))
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Generate vector of first 16 dates
#  first_n_dates <- truth$date %>% unique() %>% sort() %>% head(16)
#  # Sample those dates
#  truth_first_n_times <- truth %>% filter(date %in% first_n_dates)
#  # Visualise
#  ggplot(truth_first_n_times, aes(x = x, y = y, fill = value)) +
#    geom_raster() +
#    facet_wrap(vars(date))
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Combine each x and y coordinate to give each location a unique ID
#  truth$loc_id <- factor(paste(truth$x, truth$y, sep = "_"))
#  
#  ggplot(truth, aes(x = date, y = value, colour = loc_id)) +
#    geom_line() +
#    theme_bw() +
#    theme(legend.position = "none")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  sample_reg <- sample_area_pointily(truth, n_samples = 25, sampling_error = 0.1,
#                                 regular = TRUE)
#  sample_ran <- sample_area_pointily(truth, n_samples = 25, sampling_error = 1,
#                                 regular = FALSE)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  set.seed(1868)
#  sample_reg <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 0.1, regular = TRUE)
#  head(sample_reg)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  set.seed(1868)
#  sample_reg <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 0.1, regular = TRUE)
#  # Make mgcv see date as a date
#  sample_reg$date_dec <- lubridate::decimal_date(sample_reg$date)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_sample_reg <- mgcv::gam(point_sample ~ t2(x, y, date_dec),
#                          data = sample_reg,
#                          family = "gaussian",
#                          method = "REML")
#  
#  gam.check(fit_sample_reg)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_sample_reg <- mgcv::gam(point_sample ~ t2(x, y, date_dec),
#                          data = sample_reg,
#                          family = "gaussian",
#                          method = "REML")
#  
#  gam.check(fit_sample_reg)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_sample_ran <- mgcv::gam(point_sample ~ t2(x, y, date_dec),
#                          data = sample_ran,
#                          family = "gaussian",
#                          method = "REML")
#  
#  gam.check(fit_sample_ran)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_sample_ran <- mgcv::gam(point_sample ~ t2(x, y, date_dec),
#                          data = sample_ran,
#                          family = "gaussian",
#                          method = "REML")
#  
#  gam.check(fit_sample_ran)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  gam.check(fit_sample_reg)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  gam.check(fit_sample_ran)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  set.seed(1868)
#  sample_reg <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 0.1, regular = TRUE)
#  set.seed(1871)
#  sample_ran <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 1, regular = FALSE)
#  
#  # Make mgcv see date as a date
#  sample_reg$date_dec <- lubridate::decimal_date(sample_reg$date)
#  sample_ran$date_dec <- lubridate::decimal_date(sample_ran$date)
#  # Add family column
#  sample_reg$fam <- 1 # Observation source 1, the regular samples
#  sample_ran$fam <- 2 # Observation source 2, the random samples
#  
#  # Combine into one dataframe
#  sample_com <- rbind(sample_reg, sample_ran)
#  # Note, we need *both* fam and fam_fac for the factor smoothing later
#  sample_com$fam_fac <- factor(sample_com$fam)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  set.seed(1868)
#  sample_reg <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 0.1, regular = TRUE)
#  set.seed(1871)
#  sample_ran <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 1, regular = FALSE)
#  
#  # Make mgcv see date as a date
#  sample_reg$date_dec <- lubridate::decimal_date(sample_reg$date)
#  sample_ran$date_dec <- lubridate::decimal_date(sample_ran$date)
#  # Add family column
#  sample_reg$fam <- 1 # Observation source 1, the regular samples
#  sample_ran$fam <- 2 # Observation source 2, the random samples
#  
#  # Combine into one dataframe
#  sample_com <- rbind(sample_reg, sample_ran)
#  # Note, we need *both* fam and fam_fac for the factor smoothing later
#  sample_com$fam_fac <- factor(sample_com$fam)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  set.seed(1868)
#  sample_reg <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 0.1, regular = TRUE)
#  set.seed(1871)
#  sample_ran <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 1, regular = FALSE)
#  
#  # Make mgcv see date as a date
#  sample_reg$date_dec <- lubridate::decimal_date(sample_reg$date)
#  sample_ran$date_dec <- lubridate::decimal_date(sample_ran$date)
#  # Add family column
#  sample_reg$fam <- 1 # Observation source 1, the regular samples
#  sample_ran$fam <- 2 # Observation source 2, the random samples
#  
#  # Combine into one dataframe
#  sample_com <- rbind(sample_reg, sample_ran)
#  # Note, we need *both* fam and fam_fac for the factor smoothing later
#  sample_com$fam_fac <- factor(sample_com$fam)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  set.seed(1868)
#  sample_reg <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 0.1, regular = TRUE)
#  set.seed(1871)
#  sample_ran <- sample_area_pointily(truth, n_samples = 25,
#                                     sampling_error = 1, regular = FALSE)
#  
#  # Make mgcv see date as a date
#  sample_reg$date_dec <- lubridate::decimal_date(sample_reg$date)
#  sample_ran$date_dec <- lubridate::decimal_date(sample_ran$date)
#  # Add family column
#  sample_reg$fam <- 1 # Observation source 1, the regular samples
#  sample_ran$fam <- 2 # Observation source 2, the random samples
#  
#  # Combine into one dataframe
#  sample_com <- rbind(sample_reg, sample_ran)
#  # Note, we need *both* fam and fam_fac for the factor smoothing later
#  sample_com$fam_fac <- factor(sample_com$fam)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_sample_ran <- mgcv::gam(point_sample ~ t2(x, y, date_dec),
#                          data = sample_ran,
#                          family = "gaussian",
#                          method = "REML")
#  
#  fit_fam_I <- mgcv::gam(point_sample ~ t2(x, y, date_dec,
#                                           by = fam_fac),
#                          data = sample_com,
#                          family = "gaussian",
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_sample_ran <- mgcv::gam(point_sample ~ t2(x, y, date_dec),
#                          data = sample_ran,
#                          family = "gaussian",
#                          method = "REML")
#  
#  fit_fam_I <- mgcv::gam(point_sample ~ t2(x, y, date_dec,
#                                           by = fam_fac),
#                          data = sample_com,
#                          family = "gaussian",
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_fam_I <- mgcv::gam(point_sample ~ t2(x, y, date_dec,
#                                           by = fam_fac),
#                          data = sample_com,
#                          family = "gaussian",
#                          method = "REML")
#  gam.check(fit_fam_I)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Need to create combined response variable
#  sample_com$resp <- cbind(sample_com$point_sample, as.numeric(sample_com$fam))

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  fit_fam_I <- mgcv::gam(point_sample ~ t2(x, y, date_dec,
#                                           by = fam_fac),
#                          data = sample_com,
#                          family = "gaussian",
#                          method = "REML")
#  
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Need to create combined response variable
#  sample_com$resp <- cbind(sample_com$point_sample, as.numeric(sample_com$fam))
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Need to create combined response variable
#  sample_com$resp <- cbind(sample_com$point_sample, as.numeric(sample_com$fam))
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  # Shared trend and shared smoothness
#  fit_fam_GS <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec, fam_fac,
#                               d=c(2, 1, 1),
#                               bs=c("ts", "ts", "re")),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Shared trend and shared smoothness
#  fit_fam_GS <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec, fam_fac,
#                               d=c(2, 1, 1),
#                               bs=c("ts", "ts", "re")),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Identical trend and smoothness
#  fit_fam_G <- mgcv::gam(point_sample ~ t2(x,y, date_dec),
#                         data = sample_com,
#                         family = "gaussian",
#                         method = "REML")
#  

## ----echo=TRUE, eval=FALSE, fig.align="center"--------------------------------
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Shared trend and shared smoothness
#  fit_fam_GS <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec, fam_fac,
#                               d=c(2, 1, 1),
#                               bs=c("ts", "ts", "re")),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Identical trend and smoothness
#  fit_fam_G <- mgcv::gam(point_sample ~ t2(x,y, date_dec),
#                         data = sample_com,
#                         family = "gaussian",
#                         method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Independent trend and smooths
#  fit_fam_I <- mgcv::gam(point_sample ~ t2(x, y, date_dec,
#                                           by = fam_fac),
#                          data = sample_com,
#                          family = "gaussian",
#                          method = "REML")
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Shared trend and shared smoothness
#  fit_fam_GS <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec, fam_fac,
#                               d=c(2, 1, 1),
#                               bs=c("ts", "ts", "re")),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Identical trend and smoothness
#  fit_fam_G <- mgcv::gam(point_sample ~ t2(x,y, date_dec),
#                         data = sample_com,
#                         family = "gaussian",
#                         method = "REML")
#  

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  # Need to create combined response variable
#  sample_com$resp <- cbind(sample_com$point_sample, as.numeric(sample_com$fam))
#  # Shared trend and different smoothness
#  fit_fam_GI <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec,
#                               d=c(2, 1),
#                               bs=c("ts", "ts"),
#                               by = fam_fac),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Shared trend and shared smoothness
#  fit_fam_GS <- mgcv::gam(resp ~ t2(x, y, date_dec) +
#                            t2(x, y, date_dec, fam_fac,
#                               d=c(2, 1, 1),
#                               bs=c("ts", "ts", "re")),
#                          data = sample_com,
#                          family = gfam(list(gaussian, gaussian)),
#                          method = "REML")
#  
#  # Identical trend and smoothness
#  fit_fam_G <- mgcv::gam(point_sample ~ t2(x,y, date_dec),
#                         data = sample_com,
#                         family = "gaussian",
#                         method = "REML")

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  gam.check(fit_fam_I)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  gam.check(fit_fam_GI)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  gam.check(fit_fam_GS)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  gam.check(fit_fam_G)

## ----echo=TRUE, eval=FALSE----------------------------------------------------
#  AIC(fit_fam_I)
#  AIC(fit_fam_GI)
#  AIC(fit_fam_GS)
#  AIC(fit_fam_G)

